package bin;

import kernel.peripherals.Peripherals.Peripheral;
import lib.CLIAppBase;

using tink.CoreApi;

@:build(macros.Binstore.includeBin("Speaker", ["speaker", "sp"]))
class Speaker extends CLIAppBase {
	public function new() {
		registerSyncSubcommand("note", (args) -> {
			if (args.length < 2) {
				handle.writeLine("Not enough args");
				return false;
			}

			var sp = Peripheral.getSpeaker(args[0]);

			var note = args[1];
			var r;

			if (args.length > 3) {
				var volume = Std.parseFloat(args[2]);
				if (volume == null) {
					handle.writeLine("Failed to parse volume");
					return false;
				}

				if (args.length > 4) {
					var pitch = Std.parseInt(args[3]);
					if (pitch == null) {
						handle.writeLine("Failed to parse pitch");
						return false;
					}

					r = sp.playNote(note, volume, pitch);
				} else {
					r = sp.playNote(note, volume);
				}
			} else {
				r = sp.playNote(note);
			}

			switch r {
				case Failure(failure):
					handle.writeLine(failure);
					return false;
				case Success(_):
					return true;
			}
		}, "<addr> <note> [pitch] [volume]");

		registerSyncSubcommand("sound", (args) -> {
			if (args.length < 2) {
				handle.writeLine("Not enough args");
				return false;
			}

			var sp = Peripheral.getSpeaker(args[0]);

			var sound = args[1];
			var r;

			if (args.length > 3) {
				var volume = Std.parseFloat(args[2]);
				if (volume == null) {
					handle.writeLine("Failed to parse volume");
					return false;
				}

				if (args.length > 4) {
					var pitch = Std.parseFloat(args[3]);
					if (pitch == null) {
						handle.writeLine("Failed to parse pitch");
						return false;
					}

					r = sp.playSound(sound, volume, pitch);
				} else {
					r = sp.playSound(sound, volume);
				}
			} else {
				r = sp.playSound(sound);
			}

			switch r {
				case Failure(failure):
					handle.writeLine(failure);
					return false;
				case Success(_):
					return true;
			}
		}, "<addr> <sound> [volume] [pitch]");
	}
}
